@@ -27,11 +27,14 @@ Wheel packages are built against specific versions of CUDA libraries
 To avoid loading wrong version, these shared libraries are manually
 preloaded.
 
-# TODO(kmaehashi) Currently cuDNN only. Support cuTENSOR and NCCL.
+# TODO(kmaehashi): Support NCCL
 
 Example of `_preload_config` is as follows:
 
 {
+    # installation source
+    'packaging': 'pip',
+
     # CUDA version string
     'cuda': '11.0',
 
@@ -39,8 +42,8 @@ Example of `_preload_config` is as follows:
         # cuDNN version string
         'version': '8.0.0',
 
-        # name of the shared library
-        'filename': 'libcudnn.so.X.Y.Z'  # or `cudnn64_X.dll` for Windows
+        # names of the shared library
+        'filenames': ['libcudnn.so.X.Y.Z']  # or `cudnn64_X.dll` for Windows
     }
 }
 
@@ -52,8 +55,8 @@ _preload_config = None
 
 _preload_libs = {
     'cudnn': None,
-    # 'nccl': None,
-    # 'cutensor': None,
+    'nccl': None,
+    'cutensor': None,
 }
 
 _preload_logs = []
@@ -172,7 +175,7 @@ def _get_cub_path():
 
     if not runtime.is_hip:
         cuda_path = get_cuda_path()
-        if os.path.isdir(os.path.join(current_dir, 'core/include/cupy/cub')):
+        if os.path.isdir(os.path.join(current_dir, '_core/include/cupy/cub')):
             _cub_path = '<bundle>'
         elif cuda_path is not None and os.path.isdir(
                 os.path.join(cuda_path, 'include/cub')):
@@ -196,10 +199,15 @@ def _setup_win32_dll_directory():
     # Setup DLL directory to load CUDA Toolkit libs and shared libraries
     # added during the build process.
     if sys.platform.startswith('win32'):
+        is_conda = ((os.environ.get('CONDA_PREFIX') is not None)
+                    or (os.environ.get('CONDA_BUILD_STATE') is not None))
         # Path to the CUDA Toolkit binaries
         cuda_path = get_cuda_path()
         if cuda_path is not None:
-            cuda_bin_path = os.path.join(cuda_path, 'bin')
+            if is_conda:
+                cuda_bin_path = cuda_path
+            else:
+                cuda_bin_path = os.path.join(cuda_path, 'bin')
         else:
             cuda_bin_path = None
             warnings.warn(
@@ -243,11 +251,11 @@ def get_cupy_cuda_lib_path():
 
     This environment variable only affects wheel installations.
 
-    Shared libraries are looked up from:
-    `$CUPY_CUDA_LIB_PATH/$CUDA_VERSION/$LIBRARY_NAME/$LIBRARY_VERSION/lib64`
-    (`bin` instead of `lib64` on Windows)
+    Shared libraries are looked up from
+    `$CUPY_CUDA_LIB_PATH/$CUDA_VER/$LIB_NAME/$LIB_VER/{lib,lib64,bin}`,
+    e.g., `~/.cupy/cuda_lib/11.2/cudnn/8.1.1/lib64/libcudnn.so.8.1.1`.
 
-    The default path is `~/.cupy/cuda_lib`.
+    The default $CUPY_CUDA_LIB_PATH is `~/.cupy/cuda_lib`.
     """
     cupy_cuda_lib_path = os.environ.get('CUPY_CUDA_LIB_PATH', None)
     if cupy_cuda_lib_path is None:
@@ -267,60 +275,98 @@ def get_preload_config():
     return _preload_config
 
 
-def _preload_libraries():
+def _can_attempt_preload(lib: str) -> bool:
+    """Returns if the preload can be attempted."""
+
+    config = get_preload_config()
+    if (config is None) or (config['packaging'] == 'conda'):
+        # We don't do preload if CuPy is installed from Conda-Forge, as we
+        # cannot guarantee the version pinned in _wheel.json, which is
+        # encoded in config[lib]['filenames'], is always available on
+        # Conda-Forge. See here for the configuration files used in
+        # Conda-Forge distributions.
+        # https://github.com/conda-forge/cupy-feedstock/blob/master/recipe/preload_config/
+        _log(f'Cannot preload {lib} as this is not a wheel installation')
+        return False
+
+    if lib not in _preload_libs:
+        raise AssertionError(f'Unknown preload library: {lib}')
+
+    if lib not in config:
+        _log(f'Preload {lib} not configured in wheel')
+        return False
+
+    if _preload_libs[lib] is not None:
+        _log(f'Preload already attempted: {lib}')
+        return False
+
+    return True
+
+
+def _preload_library(lib):
     """Preload dependent shared libraries.
 
     The preload configuration file (cupy/.data/_wheel.json) will be added
     during the wheel build process.
     """
 
-    config = get_preload_config()
-    if config is None:
-        _log('Skip preloading as this is not a wheel installation')
-        return
+    _log(f'Preloading triggered for library: {lib}')
 
+    if not _can_attempt_preload(lib):
+        return
+    _preload_libs[lib] = {}
+
+    config = get_preload_config()
     cuda_version = config['cuda']
     _log('CuPy wheel package built for CUDA {}'.format(cuda_version))
 
     cupy_cuda_lib_path = get_cupy_cuda_lib_path()
     _log('CuPy CUDA library directory: {}'.format(cupy_cuda_lib_path))
 
-    for lib in _preload_libs.keys():
-        if lib not in config:
-            _log('Not preloading {}'.format(lib))
-            continue
-        version = config[lib]['version']
-        filename = config[lib]['filename']
-        _log('Looking for {} version {} ({})'.format(lib, version, filename))
+    version = config[lib]['version']
+    filenames = config[lib]['filenames']
+    for filename in filenames:
+        _log(f'Looking for {lib} version {version} ({filename})')
+
+        # "lib": cuTENSOR (Linux/Windows) / NCCL (Linux)
+        # "lib64": cuDNN (Linux)
+        # "bin": cuDNN (Windows)
+        libpath_cands = [
+            os.path.join(
+                cupy_cuda_lib_path, config['cuda'], lib, version, x,
+                filename)
+            for x in ['lib', 'lib64', 'bin']]
+        for libpath in libpath_cands:
+            if not os.path.exists(libpath):
+                _log('Rejected candidate (not found): {}'.format(libpath))
+                continue
 
-        lib64dir = 'bin' if sys.platform.startswith('win32') else 'lib64'
-        libpath = os.path.join(
-            cupy_cuda_lib_path, config['cuda'], lib, version, lib64dir,
-            filename)
-        if os.path.exists(libpath):
-            _log('Trying to load {}'.format(libpath))
             try:
+                _log(f'Trying to load {libpath}')
                 # Keep reference to the preloaded module.
-                _preload_libs[lib] = (libpath, ctypes.CDLL(libpath))
+                _preload_libs[lib][libpath] = ctypes.CDLL(libpath)
                 _log('Loaded')
+                break
             except Exception as e:
-                msg = 'CuPy failed to preload library ({}): {} ({})'.format(
-                    libpath, type(e).__name__, str(e))
+                e_type = type(e).__name__  # NOQA
+                msg = (
+                    f'CuPy failed to preload library ({libpath}): '
+                    f'{e_type} ({e})')
                 _log(msg)
                 warnings.warn(msg)
         else:
-            _log('File {} could not be found'.format(libpath))
+            _log('File {} could not be found'.format(filename))
 
             # Lookup library with fully-qualified version (e.g.,
             # `libcudnn.so.X.Y.Z`).
-            _log('Trying to load {}'.format(filename))
+            _log(f'Trying to load {filename} from default search path')
             try:
-                _preload_libs[lib] = (filename, ctypes.CDLL(filename))
+                _preload_libs[lib][filename] = ctypes.CDLL(filename)
                 _log('Loaded')
             except Exception as e:
                 # Fallback to the standard shared library lookup which only
                 # uses the major version (e.g., `libcudnn.so.X`).
-                _log('Library {} could not be preloaded: {}'.format(lib, e))
+                _log(f'Library {lib} could not be preloaded: {e}')
 
 
 def _get_preload_logs():
@@ -330,12 +376,102 @@ def _get_preload_logs():
 def _preload_warning(lib, exc):
     config = get_preload_config()
     if config is not None and lib in config:
-        warnings.warn('''
+        msg = '''
 {lib} library could not be loaded.
 
 Reason: {exc_type} ({exc})
 
 You can install the library by:
+'''
+        if config['packaging'] == 'pip':
+            msg += '''
   $ python -m cupyx.tools.install_library --library {lib} --cuda {cuda}
-'''.format(lib=lib, exc_type=type(exc).__name__, exc=str(exc),
-           cuda=config['cuda']))
+'''
+        elif config['packaging'] == 'conda':
+            msg += '''
+  $ conda install -c conda-forge {lib}
+'''
+        else:
+            raise AssertionError
+        msg = msg.format(
+            lib=lib, exc_type=type(exc).__name__, exc=str(exc),
+            cuda=config['cuda'])
+        warnings.warn(msg)
+
+
+def _detect_duplicate_installation():
+    # importlib.metadata only available in Python 3.8+.
+    if sys.version_info < (3, 8):
+        return
+    import importlib.metadata
+
+    # List of all CuPy packages, including out-dated ones.
+    known = [
+        'cupy',
+        'cupy-cuda80',
+        'cupy-cuda90',
+        'cupy-cuda91',
+        'cupy-cuda92',
+        'cupy-cuda100',
+        'cupy-cuda101',
+        'cupy-cuda102',
+        'cupy-cuda110',
+        'cupy-cuda111',
+        'cupy-cuda112',
+        'cupy-cuda113',
+        'cupy-cuda114',
+        'cupy-cuda115',
+        'cupy-cuda116',
+        'cupy-cuda117',
+        'cupy-cuda11x',
+        'cupy-rocm-4-0',
+        'cupy-rocm-4-1',
+        'cupy-rocm-4-2',
+        'cupy-rocm-4-3',
+        'cupy-rocm-5-0',
+    ]
+    cupy_installed = [
+        name for name in known
+        if list(importlib.metadata.distributions(name=name))]
+    if 1 < len(cupy_installed):
+        cupy_packages_list = ', '.join(sorted(cupy_installed))
+        warnings.warn(f'''
+--------------------------------------------------------------------------------
+
+  CuPy may not function correctly because multiple CuPy packages are installed
+  in your environment:
+
+    {cupy_packages_list}
+
+  Follow these steps to resolve this issue:
+
+    1. For all packages listed above, run the following command to remove all
+       existing CuPy installations:
+
+         $ pip uninstall <package_name>
+
+      If you previously installed CuPy via conda, also run the following:
+
+         $ conda uninstall cupy
+
+    2. Install the appropriate CuPy package.
+       Refer to the Installation Guide for detailed instructions.
+
+         https://docs.cupy.dev/en/stable/install.html
+
+--------------------------------------------------------------------------------
+''')
+
+
+def _diagnose_import_error() -> str:
+    # TODO(kmaehashi): provide better diagnostics.
+    return '''\
+Failed to import CuPy.
+
+If you installed CuPy via wheels (cupy-cudaXXX or cupy-rocm-X-X), make sure that the package matches with the version of CUDA or ROCm installed.
+
+On Linux, you may need to set LD_LIBRARY_PATH environment variable depending on how you installed CUDA/ROCm.
+On Windows, try setting CUDA_PATH environment variable.
+
+Check the Installation Guide for details:
+  https://docs.cupy.dev/en/latest/install.html'''  # NOQA
