@@ -71,6 +71,10 @@ cudaError_t cudaDeviceEnablePeerAccess(int peerDeviceId, unsigned int flags) {
     return hipDeviceEnablePeerAccess(peerDeviceId, flags);
 }
 
+cudaError_t cudaDeviceDisablePeerAccess(int peerDeviceId) {
+    return hipDeviceDisablePeerAccess(peerDeviceId);
+}
+
 cudaError_t cudaDeviceGetLimit(size_t* pValue, cudaLimit limit) {
     return hipDeviceGetLimit(pValue, limit);
 }
@@ -110,6 +114,21 @@ cudaError_t cudaIpcOpenMemHandle(void** devPtr, cudaIpcMemHandle_t handle, unsig
 }
 
 // Memory management
+enum cudaMemAllocationType {};  // stub
+enum cudaMemAllocationHandleType {};  // stub
+enum cudaMemLocationType {};  // stub
+struct cudaMemLocation {  // stub
+    int id;
+    cudaMemLocationType type;
+};
+struct cudaMemPoolProps {  // stub
+    cudaMemAllocationType allocType;
+    cudaMemAllocationHandleType handleTypes;
+    struct cudaMemLocation location;
+    unsigned char reserved[64];
+    void* win32SecurityAttributes;
+};
+
 cudaError_t cudaMalloc(void** ptr, size_t size) {
     return hipMalloc(ptr, size);
 }
@@ -138,8 +157,12 @@ cudaError_t cudaHostUnregister(...) {
     return hipErrorUnknown;
 }
 
-cudaError_t cudaMallocManaged(...) {
+cudaError_t cudaMallocManaged(void** ptr, size_t size, unsigned int flags) {
+#if HIP_VERSION >= 40300000
+    return hipMallocManaged(ptr, size, flags);
+#else
     return hipErrorUnknown;
+#endif
 }
 
 int cudaFree(void* ptr) {
@@ -225,12 +248,22 @@ cudaError_t cudaMemsetAsync(void* dst, int value, size_t sizeBytes,
     return hipMemsetAsync(dst, value, sizeBytes, stream);
 }
 
-cudaError_t cudaMemAdvise(...) {
+cudaError_t cudaMemAdvise(const void *devPtr, size_t count,
+                          cudaMemoryAdvise advice, int device) {
+#if HIP_VERSION >= 40300000
+    return hipMemAdvise(devPtr, count, advice, device);
+#else
     return hipErrorUnknown;
+#endif
 }
 
-cudaError_t cudaMemPrefetchAsync(...) {
+cudaError_t cudaMemPrefetchAsync(const void *devPtr, size_t count,
+				 int dstDevice, cudaStream_t stream) {
+#if HIP_VERSION >= 40300000
+    return hipMemPrefetchAsync(devPtr, count, dstDevice, stream);
+#else
     return hipErrorUnknown;
+#endif
 }
 
 cudaError_t cudaPointerGetAttributes(cudaPointerAttributes *attributes,
@@ -257,6 +290,18 @@ cudaError_t cudaGetDeviceProperties(cudaDeviceProp *prop, int device) {
     return hipGetDeviceProperties(prop, device);
 }
 
+cudaError_t cudaMallocFromPoolAsync(...) {
+    return hipErrorUnknown;
+}
+
+cudaError_t cudaMemPoolCreate(...) {
+    return hipErrorUnknown;
+}
+
+cudaError_t cudaMemPoolDestroy(...) {
+    return hipErrorUnknown;
+}
+
 cudaError_t cudaDeviceGetDefaultMemPool(...) {
     return hipErrorUnknown;
 }
@@ -273,8 +318,24 @@ cudaError_t cudaMemPoolTrimTo(...) {
     return hipErrorUnknown;
 }
 
+cudaError_t cudaMemPoolGetAttribute(...) {
+    return hipErrorUnknown;
+}
+
+cudaError_t cudaMemPoolSetAttribute(...) {
+    return hipErrorUnknown;
+}
+
 
 // Stream and Event
+#if HIP_VERSION >= 40300000
+typedef hipStreamCaptureMode cudaStreamCaptureMode;
+typedef hipStreamCaptureStatus cudaStreamCaptureStatus;
+#else
+enum cudaStreamCaptureMode {};
+enum cudaStreamCaptureStatus {};
+#endif
+
 cudaError_t cudaStreamCreate(cudaStream_t *stream) {
     return hipStreamCreate(stream);
 }
@@ -340,6 +401,32 @@ cudaError_t cudaEventSynchronize(cudaEvent_t event) {
     return hipEventSynchronize(event);
 }
 
+cudaError_t cudaStreamBeginCapture(cudaStream_t stream,
+                                   cudaStreamCaptureMode mode) {
+#if HIP_VERSION >= 40300000
+    return hipStreamBeginCapture(stream, mode);
+#else
+    return hipErrorUnknown;
+#endif
+}
+
+cudaError_t cudaStreamEndCapture(cudaStream_t stream, cudaGraph_t* pGraph) {
+#if HIP_VERSION >= 40300000
+    return hipStreamEndCapture(stream, pGraph);
+#else
+    return hipErrorUnknown;
+#endif
+}
+
+cudaError_t cudaStreamIsCapturing(cudaStream_t stream,
+                                  cudaStreamCaptureStatus* pCaptureStatus) {
+#if HIP_VERSION >= 50000000
+    return hipStreamIsCapturing(stream, pCaptureStatus);
+#else
+    return hipErrorUnknown;
+#endif
+}
+
 
 // Texture
 cudaError_t cudaCreateTextureObject(...) {
@@ -387,6 +474,48 @@ cudaError_t cudaDestroySurfaceObject(cudaSurfaceObject_t surfObject) {
     return hipDestroySurfaceObject(surfObject);
 }
 
+// CUDA Graph
+cudaError_t cudaGraphInstantiate(
+	cudaGraphExec_t* pGraphExec,
+	cudaGraph_t graph,
+	cudaGraphNode_t* pErrorNode,
+	char* pLogBuffer,
+	size_t bufferSize) {
+#if HIP_VERSION >= 40300000
+    return hipGraphInstantiate(pGraphExec, graph, pErrorNode, pLogBuffer, bufferSize);
+#else
+    return hipErrorUnknown;
+#endif
+}
+
+cudaError_t cudaGraphDestroy(cudaGraph_t graph) {
+#if HIP_VERSION >= 40300000
+    return hipGraphDestroy(graph);
+#else
+    return hipErrorUnknown;
+#endif
+}
+
+cudaError_t cudaGraphExecDestroy(cudaGraphExec_t graphExec) {
+#if HIP_VERSION >= 40300000
+    return hipGraphExecDestroy(graphExec);
+#else
+    return hipErrorUnknown;
+#endif
+}
+
+cudaError_t cudaGraphLaunch(cudaGraphExec_t graphExec, cudaStream_t stream) {
+#if HIP_VERSION >= 40300000
+    return hipGraphLaunch(graphExec, stream);
+#else
+    return hipErrorUnknown;
+#endif
+}
+
+cudaError_t cudaGraphUpload(...) {
+    return hipErrorUnknown;
+}
+
 } // extern "C"
 
 #endif // #ifndef INCLUDE_GUARD_HIP_CUPY_RUNTIME_H
