@@ -1,15 +1,13 @@
 import operator
-import unittest
 
 import numpy
-import six
+import pytest
 
 import cupy
 from cupy import testing
 
 
-@testing.gpu
-class TestArrayElementwiseOp(unittest.TestCase):
+class TestArrayElementwiseOp:
 
     @testing.for_all_dtypes_combination(names=['x_type', 'y_type'])
     @testing.numpy_cupy_allclose(rtol=1e-6, accept_error=TypeError)
@@ -33,7 +31,6 @@ class TestArrayElementwiseOp(unittest.TestCase):
     def test_radd_scalar(self):
         self.check_array_scalar_op(operator.add, swap=True)
 
-    @testing.with_requires('numpy>=1.10')
     def test_iadd_scalar(self):
         self.check_array_scalar_op(operator.iadd)
 
@@ -43,7 +40,6 @@ class TestArrayElementwiseOp(unittest.TestCase):
     def test_rsub_scalar(self):
         self.check_array_scalar_op(operator.sub, swap=True, no_bool=True)
 
-    @testing.with_requires('numpy>=1.10')
     def test_isub_scalar(self):
         self.check_array_scalar_op(operator.isub, no_bool=True)
 
@@ -53,54 +49,32 @@ class TestArrayElementwiseOp(unittest.TestCase):
     def test_rmul_scalar(self):
         self.check_array_scalar_op(operator.mul, swap=True)
 
-    @testing.with_requires('numpy>=1.10')
     def test_imul_scalar(self):
         self.check_array_scalar_op(operator.imul)
 
     def test_truediv_scalar(self):
-        with testing.NumpyError(divide='ignore'):
+        with numpy.errstate(divide='ignore'):
             self.check_array_scalar_op(operator.truediv)
 
     def test_rtruediv_scalar(self):
-        with testing.NumpyError(divide='ignore'):
+        with numpy.errstate(divide='ignore'):
             self.check_array_scalar_op(operator.truediv, swap=True)
 
-    @testing.with_requires('numpy>=1.10')
     def test_itruediv_scalar(self):
-        with testing.NumpyError(divide='ignore'):
+        with numpy.errstate(divide='ignore'):
             self.check_array_scalar_op(operator.itruediv)
 
-    def test_div_scalar(self):
-        if six.PY3:
-            return
-        with testing.NumpyError(divide='ignore'):
-            self.check_array_scalar_op(operator.div)
-
-    def test_rdiv_scalar(self):
-        if six.PY3:
-            return
-        with testing.NumpyError(divide='ignore'):
-            self.check_array_scalar_op(operator.div, swap=True)
-
-    @testing.with_requires('numpy>=1.10')
-    def test_idiv_scalar(self):
-        if six.PY3:
-            return
-        with testing.NumpyError(divide='ignore'):
-            self.check_array_scalar_op(operator.idiv)
-
     def test_floordiv_scalar(self):
-        with testing.NumpyError(divide='ignore'):
+        with numpy.errstate(divide='ignore'):
             self.check_array_scalar_op(operator.floordiv, no_complex=True)
 
     def test_rfloordiv_scalar(self):
-        with testing.NumpyError(divide='ignore'):
+        with numpy.errstate(divide='ignore'):
             self.check_array_scalar_op(operator.floordiv, swap=True,
                                        no_complex=True)
 
-    @testing.with_requires('numpy>=1.10')
     def test_ifloordiv_scalar(self):
-        with testing.NumpyError(divide='ignore'):
+        with numpy.errstate(divide='ignore'):
             self.check_array_scalar_op(operator.ifloordiv, no_complex=True)
 
     def test_pow_scalar(self):
@@ -115,27 +89,26 @@ class TestArrayElementwiseOp(unittest.TestCase):
         a = xp.array([[1, 2, 3], [4, 5, 6]], x_type)
         return operator.ipow(a, y_type(3))
 
-    @testing.with_requires('numpy>=1.10')
     def test_ipow_scalar(self):
         self.check_ipow_scalar()
 
     def test_divmod0_scalar(self):
-        with testing.NumpyError(divide='ignore'):
+        with numpy.errstate(divide='ignore'):
             self.check_array_scalar_op(lambda x, y: divmod(x, y)[0],
                                        no_complex=True)
 
     def test_divmod1_scalar(self):
-        with testing.NumpyError(divide='ignore'):
+        with numpy.errstate(divide='ignore'):
             self.check_array_scalar_op(lambda x, y: divmod(x, y)[1],
                                        no_complex=True)
 
     def test_rdivmod0_scalar(self):
-        with testing.NumpyError(divide='ignore'):
+        with numpy.errstate(divide='ignore'):
             self.check_array_scalar_op(lambda x, y: divmod(x, y)[0], swap=True,
                                        no_complex=True)
 
     def test_rdivmod1_scalar(self):
-        with testing.NumpyError(divide='ignore'):
+        with numpy.errstate(divide='ignore'):
             self.check_array_scalar_op(lambda x, y: divmod(x, y)[1], swap=True,
                                        no_complex=True)
 
@@ -174,59 +147,53 @@ class TestArrayElementwiseOp(unittest.TestCase):
     def test_add_array(self):
         self.check_array_array_op(operator.add)
 
-    @testing.with_requires('numpy>=1.10')
     def test_iadd_array(self):
         self.check_array_array_op(operator.iadd)
 
     def test_sub_array(self):
         self.check_array_array_op(operator.sub, no_bool=True)
 
-    @testing.with_requires('numpy>=1.10')
     def test_isub_array(self):
         self.check_array_array_op(operator.isub, no_bool=True)
 
     def test_mul_array(self):
         self.check_array_array_op(operator.mul)
 
-    @testing.with_requires('numpy>=1.10')
     def test_imul_array(self):
         self.check_array_array_op(operator.imul)
 
     def test_truediv_array(self):
-        with testing.NumpyError(divide='ignore'):
+        with numpy.errstate(divide='ignore'):
             self.check_array_array_op(operator.truediv)
 
-    @testing.with_requires('numpy>=1.10')
     def test_itruediv_array(self):
-        with testing.NumpyError(divide='ignore'):
+        with numpy.errstate(divide='ignore'):
             self.check_array_array_op(operator.itruediv)
 
-    def test_div_array(self):
-        if six.PY3:
-            return
-        with testing.NumpyError(divide='ignore'):
-            self.check_array_array_op(operator.div)
-
-    @testing.with_requires('numpy>=1.10')
-    def test_idiv_array(self):
-        if six.PY3:
-            return
-        with testing.NumpyError(divide='ignore'):
-            self.check_array_array_op(operator.idiv)
-
     def test_floordiv_array(self):
-        with testing.NumpyError(divide='ignore'):
+        with numpy.errstate(divide='ignore'):
             self.check_array_array_op(operator.floordiv, no_complex=True)
 
-    @testing.with_requires('numpy>=1.10')
     def test_ifloordiv_array(self):
         if '1.16.1' <= numpy.lib.NumpyVersion(numpy.__version__) < '1.18.0':
             self.skipTest("NumPy Issue #12927")
-        with testing.NumpyError(divide='ignore'):
+        with numpy.errstate(divide='ignore'):
             self.check_array_array_op(operator.ifloordiv, no_complex=True)
 
+    @testing.for_all_dtypes_combination(names=['x_type', 'y_type'])
+    @testing.numpy_cupy_allclose(atol=1e-5, rtol=1e-6, accept_error=TypeError)
+    def check_pow_array(self, xp, x_type, y_type):
+        a = xp.array([[1, 2, 3], [4, 5, 6]], x_type)
+        b = xp.array([[6, 5, 4], [3, 2, 1]], y_type)
+        return operator.pow(a, b)
+
     def test_pow_array(self):
-        self.check_array_array_op(operator.pow)
+        # There are some precission issues in HIP that prevent
+        # checking with atol=0
+        if cupy.cuda.runtime.is_hip:
+            self.check_pow_array()
+        else:
+            self.check_array_array_op(operator.pow)
 
     @testing.for_all_dtypes_combination(names=['x_type', 'y_type'])
     @testing.numpy_cupy_allclose(atol=1.0, accept_error=TypeError)
@@ -235,16 +202,15 @@ class TestArrayElementwiseOp(unittest.TestCase):
         b = xp.array([[6, 5, 4], [3, 2, 1]], y_type)
         return operator.ipow(a, b)
 
-    @testing.with_requires('numpy>=1.10')
     def test_ipow_array(self):
         self.check_ipow_array()
 
     def test_divmod0_array(self):
-        with testing.NumpyError(divide='ignore'):
+        with numpy.errstate(divide='ignore'):
             self.check_array_array_op(lambda x, y: divmod(x, y)[0])
 
     def test_divmod1_array(self):
-        with testing.NumpyError(divide='ignore'):
+        with numpy.errstate(divide='ignore'):
             self.check_array_array_op(lambda x, y: divmod(x, y)[1])
 
     def test_lt_array(self):
@@ -282,7 +248,6 @@ class TestArrayElementwiseOp(unittest.TestCase):
     def test_broadcasted_add(self):
         self.check_array_broadcasted_op(operator.add)
 
-    @testing.with_requires('numpy>=1.10')
     def test_broadcasted_iadd(self):
         self.check_array_broadcasted_op(operator.iadd)
 
@@ -290,7 +255,6 @@ class TestArrayElementwiseOp(unittest.TestCase):
         # TODO(unno): sub for boolean array is deprecated in numpy>=1.13
         self.check_array_broadcasted_op(operator.sub, no_bool=True)
 
-    @testing.with_requires('numpy>=1.10')
     def test_broadcasted_isub(self):
         # TODO(unno): sub for boolean array is deprecated in numpy>=1.13
         self.check_array_broadcasted_op(operator.isub, no_bool=True)
@@ -298,48 +262,43 @@ class TestArrayElementwiseOp(unittest.TestCase):
     def test_broadcasted_mul(self):
         self.check_array_broadcasted_op(operator.mul)
 
-    @testing.with_requires('numpy>=1.10')
     def test_broadcasted_imul(self):
         self.check_array_broadcasted_op(operator.imul)
 
     def test_broadcasted_truediv(self):
-        with testing.NumpyError(divide='ignore'):
+        with numpy.errstate(divide='ignore'):
             self.check_array_broadcasted_op(operator.truediv)
 
-    @testing.with_requires('numpy>=1.10')
     def test_broadcasted_itruediv(self):
-        with testing.NumpyError(divide='ignore'):
+        with numpy.errstate(divide='ignore'):
             self.check_array_broadcasted_op(operator.itruediv)
 
-    def test_broadcasted_div(self):
-        if six.PY3:
-            return
-        with testing.NumpyError(divide='ignore'):
-            self.check_array_broadcasted_op(operator.div)
-
-    @testing.with_requires('numpy>=1.10')
-    def test_broadcasted_idiv(self):
-        if six.PY3:
-            return
-        with testing.NumpyError(divide='ignore'):
-            self.check_array_broadcasted_op(operator.idiv)
-
     def test_broadcasted_floordiv(self):
-        with testing.NumpyError(divide='ignore'):
+        with numpy.errstate(divide='ignore'):
             self.check_array_broadcasted_op(operator.floordiv, no_complex=True)
 
-    @testing.with_requires('numpy>=1.10')
     def test_broadcasted_ifloordiv(self):
         if '1.16.1' <= numpy.lib.NumpyVersion(numpy.__version__) < '1.18.0':
             self.skipTest("NumPy Issue #12927")
-        with testing.NumpyError(divide='ignore'):
+        with numpy.errstate(divide='ignore'):
             self.check_array_broadcasted_op(operator.ifloordiv,
                                             no_complex=True)
 
+    @testing.for_all_dtypes_combination(names=['x_type', 'y_type'])
+    @testing.numpy_cupy_allclose(atol=1e-5, rtol=1e-6, accept_error=TypeError)
+    def check_broadcasted_pow(self, xp, x_type, y_type):
+        a = xp.array([[1, 2, 3], [4, 5, 6]], x_type)
+        b = xp.array([[1], [2]], y_type)
+        return operator.pow(a, b)
+
     def test_broadcasted_pow(self):
-        self.check_array_broadcasted_op(operator.pow)
+        # There are some precission issues in HIP that prevent
+        # checking with atol=0
+        if cupy.cuda.runtime.is_hip:
+            self.check_broadcasted_pow()
+        else:
+            self.check_array_broadcasted_op(operator.pow)
 
-    @testing.with_requires('numpy>=1.10')
     @testing.for_all_dtypes_combination(names=['x_type', 'y_type'])
     @testing.numpy_cupy_allclose(atol=1.0, accept_error=TypeError)
     def check_broadcasted_ipow(self, xp, x_type, y_type):
@@ -347,17 +306,16 @@ class TestArrayElementwiseOp(unittest.TestCase):
         b = xp.array([[1], [2]], y_type)
         return operator.ipow(a, b)
 
-    @testing.with_requires('numpy>=1.10')
     def test_broadcasted_ipow(self):
         self.check_broadcasted_ipow()
 
     def test_broadcasted_divmod0(self):
-        with testing.NumpyError(divide='ignore'):
+        with numpy.errstate(divide='ignore'):
             self.check_array_broadcasted_op(lambda x, y: divmod(x, y)[0],
                                             no_complex=True)
 
     def test_broadcasted_divmod1(self):
-        with testing.NumpyError(divide='ignore'):
+        with numpy.errstate(divide='ignore'):
             self.check_array_broadcasted_op(lambda x, y: divmod(x, y)[1],
                                             no_complex=True)
 
@@ -380,7 +338,7 @@ class TestArrayElementwiseOp(unittest.TestCase):
         self.check_array_broadcasted_op(operator.ne)
 
     @testing.for_all_dtypes_combination(names=['x_type', 'y_type'])
-    @testing.numpy_cupy_allclose()
+    @testing.numpy_cupy_allclose(rtol=1e-6)
     def check_array_doubly_broadcasted_op(self, op, xp, x_type, y_type,
                                           no_bool=False, no_complex=False):
         x_dtype = numpy.dtype(x_type)
@@ -403,31 +361,25 @@ class TestArrayElementwiseOp(unittest.TestCase):
         self.check_array_doubly_broadcasted_op(operator.mul)
 
     def test_doubly_broadcasted_truediv(self):
-        with testing.NumpyError(divide='ignore', invalid='ignore'):
+        with numpy.errstate(divide='ignore', invalid='ignore'):
             self.check_array_doubly_broadcasted_op(operator.truediv)
 
     def test_doubly_broadcasted_floordiv(self):
-        with testing.NumpyError(divide='ignore'):
+        with numpy.errstate(divide='ignore'):
             self.check_array_doubly_broadcasted_op(operator.floordiv,
                                                    no_complex=True)
 
-    def test_doubly_broadcasted_div(self):
-        if six.PY3:
-            return
-        with testing.NumpyError(divide='ignore'):
-            self.check_array_doubly_broadcasted_op(operator.div)
-
     def test_doubly_broadcasted_pow(self):
         self.check_array_doubly_broadcasted_op(operator.pow)
 
     def test_doubly_broadcasted_divmod0(self):
-        with testing.NumpyError(divide='ignore'):
+        with numpy.errstate(divide='ignore'):
             self.check_array_doubly_broadcasted_op(
                 lambda x, y: divmod(x, y)[0],
                 no_complex=True)
 
     def test_doubly_broadcasted_divmod1(self):
-        with testing.NumpyError(divide='ignore'):
+        with numpy.errstate(divide='ignore'):
             self.check_array_doubly_broadcasted_op(
                 lambda x, y: divmod(x, y)[1],
                 no_complex=True)
@@ -470,17 +422,14 @@ class TestArrayElementwiseOp(unittest.TestCase):
 
     @testing.for_all_dtypes(no_bool=True)
     def check_typecast(self, val, dtype):
-        operators = [operator.add, operator.sub, operator.mul]
-        if six.PY3:
-            operators.append(operator.truediv)
-        else:
-            operators.append(operator.div)
+        operators = [
+            operator.add, operator.sub, operator.mul, operator.truediv]
 
         for op in operators:
-            with testing.NumpyError(divide='ignore', invalid='ignore'):
+            with numpy.errstate(divide='ignore', invalid='ignore'):
                 a = op(val, (testing.shaped_arange((5,), numpy, dtype) - 2))
             b = op(val, (testing.shaped_arange((5,), cupy, dtype) - 2))
-            self.assertEqual(a.dtype, b.dtype)
+            assert a.dtype == b.dtype
 
     def test_typecast_bool1(self):
         self.check_typecast(True)
@@ -515,9 +464,23 @@ class TestArrayElementwiseOp(unittest.TestCase):
     def test_typecast_float2(self):
         self.check_typecast(100000.0)
 
+    # Skip float16 because of NumPy #19514
+    @testing.for_all_dtypes(name='x_type', no_float16=True)
+    @testing.numpy_cupy_allclose()
+    def check_array_boolarray_op(self, op, xp, x_type):
+        a = xp.array([[2, 7, 1], [8, 2, 8]], x_type)
+        # Cast from np.bool8 array should not read bytes
+        b = xp.array([[3, 1, 4], [-1, -5, -9]], numpy.int8).view(bool)
+        return op(a, b)
 
-@testing.gpu
-class TestArrayIntElementwiseOp(unittest.TestCase):
+    def test_add_array_boolarray(self):
+        self.check_array_boolarray_op(operator.add)
+
+    def test_iadd_array_boolarray(self):
+        self.check_array_boolarray_op(operator.iadd)
+
+
+class TestArrayIntElementwiseOp:
 
     @testing.for_all_dtypes_combination(names=['x_type', 'y_type'])
     @testing.numpy_cupy_allclose(accept_error=TypeError)
@@ -559,11 +522,11 @@ class TestArrayIntElementwiseOp(unittest.TestCase):
         self.check_array_scalar_op(operator.xor, swap=True)
 
     def test_mod_scalar(self):
-        with testing.NumpyError(divide='ignore', invalid='ignore'):
+        with numpy.errstate(divide='ignore', invalid='ignore'):
             self.check_array_scalar_op(operator.mod)
 
     def test_rmod_scalar(self):
-        with testing.NumpyError(divide='ignore', invalid='ignore'):
+        with numpy.errstate(divide='ignore', invalid='ignore'):
             self.check_array_scalar_op(operator.mod, swap=True)
 
     @testing.for_all_dtypes_combination(names=['x_type', 'y_type'])
@@ -606,11 +569,11 @@ class TestArrayIntElementwiseOp(unittest.TestCase):
         self.check_array_scalarzero_op(operator.xor, swap=True)
 
     def test_mod_scalarzero(self):
-        with testing.NumpyError(divide='ignore', invalid='ignore'):
+        with numpy.errstate(divide='ignore', invalid='ignore'):
             self.check_array_scalarzero_op(operator.mod)
 
     def test_rmod_scalarzero(self):
-        with testing.NumpyError(divide='ignore', invalid='ignore'):
+        with numpy.errstate(divide='ignore', invalid='ignore'):
             self.check_array_scalarzero_op(operator.mod, swap=True)
 
     @testing.for_all_dtypes_combination(names=['x_type', 'y_type'])
@@ -623,45 +586,39 @@ class TestArrayIntElementwiseOp(unittest.TestCase):
     def test_lshift_array(self):
         self.check_array_array_op(operator.lshift)
 
-    @testing.with_requires('numpy>=1.10')
     def test_ilshift_array(self):
         self.check_array_array_op(operator.ilshift)
 
     def test_rshift_array(self):
         self.check_array_array_op(operator.rshift)
 
-    @testing.with_requires('numpy>=1.10')
     def test_irshift_array(self):
         self.check_array_array_op(operator.irshift)
 
     def test_and_array(self):
         self.check_array_array_op(operator.and_)
 
-    @testing.with_requires('numpy>=1.10')
     def test_iand_array(self):
         self.check_array_array_op(operator.iand)
 
     def test_or_array(self):
         self.check_array_array_op(operator.or_)
 
-    @testing.with_requires('numpy>=1.10')
     def test_ior_array(self):
         self.check_array_array_op(operator.ior)
 
     def test_xor_array(self):
         self.check_array_array_op(operator.xor)
 
-    @testing.with_requires('numpy>=1.10')
     def test_ixor_array(self):
         self.check_array_array_op(operator.ixor)
 
     def test_mod_array(self):
-        with testing.NumpyError(divide='ignore', invalid='ignore'):
+        with numpy.errstate(divide='ignore', invalid='ignore'):
             self.check_array_array_op(operator.mod)
 
-    @testing.with_requires('numpy>=1.10')
     def test_imod_array(self):
-        with testing.NumpyError(divide='ignore', invalid='ignore'):
+        with numpy.errstate(divide='ignore', invalid='ignore'):
             self.check_array_array_op(operator.imod)
 
     @testing.for_all_dtypes_combination(names=['x_type', 'y_type'])
@@ -674,45 +631,39 @@ class TestArrayIntElementwiseOp(unittest.TestCase):
     def test_broadcasted_lshift(self):
         self.check_array_broadcasted_op(operator.lshift)
 
-    @testing.with_requires('numpy>=1.10')
     def test_broadcasted_ilshift(self):
         self.check_array_broadcasted_op(operator.ilshift)
 
     def test_broadcasted_rshift(self):
         self.check_array_broadcasted_op(operator.rshift)
 
-    @testing.with_requires('numpy>=1.10')
     def test_broadcasted_irshift(self):
         self.check_array_broadcasted_op(operator.irshift)
 
     def test_broadcasted_and(self):
         self.check_array_broadcasted_op(operator.and_)
 
-    @testing.with_requires('numpy>=1.10')
     def test_broadcasted_iand(self):
         self.check_array_broadcasted_op(operator.iand)
 
     def test_broadcasted_or(self):
         self.check_array_broadcasted_op(operator.or_)
 
-    @testing.with_requires('numpy>=1.10')
     def test_broadcasted_ior(self):
         self.check_array_broadcasted_op(operator.ior)
 
     def test_broadcasted_xor(self):
         self.check_array_broadcasted_op(operator.xor)
 
-    @testing.with_requires('numpy>=1.10')
     def test_broadcasted_ixor(self):
         self.check_array_broadcasted_op(operator.ixor)
 
     def test_broadcasted_mod(self):
-        with testing.NumpyError(divide='ignore', invalid='ignore'):
+        with numpy.errstate(divide='ignore', invalid='ignore'):
             self.check_array_broadcasted_op(operator.mod)
 
-    @testing.with_requires('numpy>=1.10')
     def test_broadcasted_imod(self):
-        with testing.NumpyError(divide='ignore', invalid='ignore'):
+        with numpy.errstate(divide='ignore', invalid='ignore'):
             self.check_array_broadcasted_op(operator.imod)
 
     @testing.for_all_dtypes_combination(names=['x_type', 'y_type'])
@@ -738,5 +689,87 @@ class TestArrayIntElementwiseOp(unittest.TestCase):
         self.check_array_doubly_broadcasted_op(operator.xor)
 
     def test_doubly_broadcasted_mod(self):
-        with testing.NumpyError(divide='ignore', invalid='ignore'):
+        with numpy.errstate(divide='ignore', invalid='ignore'):
             self.check_array_doubly_broadcasted_op(operator.mod)
+
+
+@pytest.mark.parametrize('value', [
+    None,
+    Ellipsis,
+    object(),
+    numpy._NoValue,
+])
+class TestArrayObjectComparison:
+
+    @pytest.mark.parametrize('swap', [False, True])
+    @testing.for_all_dtypes()
+    @testing.numpy_cupy_array_equal()
+    def test_eq_object(self, xp, dtype, value, swap):
+        a = xp.array([[1, 2, 3], [4, 5, 6]], dtype=dtype)
+        if swap:
+            return value == a
+        else:
+            return a == value
+
+    @pytest.mark.parametrize('swap', [False, True])
+    @testing.for_all_dtypes()
+    @testing.numpy_cupy_array_equal()
+    def test_ne_object(self, xp, dtype, value, swap):
+        a = xp.array([[1, 2, 3], [4, 5, 6]], dtype=dtype)
+        if swap:
+            return value != a
+        else:
+            return a != value
+
+
+class HasEq:
+    def __eq__(self, other):
+        return (other == 2) | (other == 4)
+
+
+class HasNe:
+    def __ne__(self, other):
+        return (other == 2) | (other == 4)
+
+
+class HasEqSub(HasEq):
+    pass
+
+
+class CustomInt(int):
+    pass
+
+
+@pytest.mark.parametrize('dtype', ['int32', 'float64'])
+@pytest.mark.parametrize('value', [
+    HasEq(),
+    HasNe(),  # eq test passes because `==` does not fall back to `__ne__`.
+    HasEqSub(),
+    CustomInt(3),
+])
+class TestArrayObjectComparisonDifficult:
+
+    # OK to raise TypeError.
+    # If CuPy returns a result, it should match with NumPy's result.
+
+    def test_eq_object(self, dtype, value):
+        expected = numpy.array([[1, 2, 3], [4, 5, 6]], dtype=dtype) == value
+
+        a = cupy.array([[1, 2, 3], [4, 5, 6]], dtype=dtype)
+        try:
+            res = a == value
+        except TypeError:
+            pytest.skip()
+
+        cupy.testing.assert_array_equal(res, expected)
+
+    def test_ne_object(self, dtype, value):
+        expected = numpy.array([[1, 2, 3], [4, 5, 6]], dtype=dtype) != value
+
+        a = cupy.array([[1, 2, 3], [4, 5, 6]], dtype=dtype)
+        try:
+            res = a != value
+        except TypeError:
+            pytest.skip()
+
+        cupy.testing.assert_array_equal(res, expected)
