@@ -1,8 +1,8 @@
 import pickle
 import unittest
+from unittest import mock
 
-import mock
-import six
+import pytest
 
 import cupy
 from cupy.cuda import compiler
@@ -13,23 +13,16 @@ def cuda_version():
     return cupy.cuda.runtime.runtimeGetVersion()
 
 
-@testing.gpu
+@unittest.skipIf(cupy.cuda.runtime.is_hip, 'CUDA specific tests')
 class TestNvrtcArch(unittest.TestCase):
     def setUp(self):
-        cupy.util.clear_memo()  # _get_arch result is cached
+        cupy.clear_memo()  # _get_arch result is cached
 
     def _check_get_arch(self, device_cc, expected_arch):
         with mock.patch('cupy.cuda.device.Device') as device_class:
             device_class.return_value.compute_capability = device_cc
             assert compiler._get_arch() == expected_arch
-        cupy.util.clear_memo()  # _get_arch result is cached
-
-    @unittest.skipUnless(cuda_version() < 9000, 'Requires CUDA 8.x or earlier')
-    def test_get_arch_cuda8(self):
-        self._check_get_arch('37', '37')
-        self._check_get_arch('50', '50')
-        self._check_get_arch('52', '52')
-        self._check_get_arch('53', '53')  # Tegra
+        cupy.clear_memo()  # _get_arch result is cached
 
     @unittest.skipUnless(9000 <= cuda_version(), 'Requires CUDA 9.x or later')
     def test_get_arch_cuda9(self):
@@ -42,25 +35,14 @@ class TestNvrtcArch(unittest.TestCase):
     def test_get_arch_cuda101(self):
         self._check_get_arch('75', '75')
 
+    @unittest.skipUnless(11000 <= cuda_version(),
+                         'Requires CUDA 11.0 or later')
+    def test_get_arch_cuda11(self):
+        self._check_get_arch('80', '80')
+
     def _compile(self, arch):
         compiler.compile_using_nvrtc('', arch=arch)
 
-    @unittest.skipUnless(cuda_version() < 9000, 'Requires CUDA 8.x or earlier')
-    def test_compile_cuda8(self):
-        # This test is intended to detect specification change in NVRTC API.
-
-        # It should not fail.
-        # (Do not test `compute_53` as it is for Tegra.)
-        self._compile('52')
-
-        # It should fail.
-        # (`compute_60` and `compute_61` are not supported by NVRTC in CUDA 8
-        #  but it does not raise error when used.)
-        self.assertRaises(
-            compiler.CompileException, self._compile, '54')
-        self.assertRaises(
-            compiler.CompileException, self._compile, '70')
-
     @unittest.skipUnless(9000 <= cuda_version(), 'Requires CUDA 9.0 or later')
     def test_compile_cuda9(self):
         # This test is intended to detect specification change in NVRTC API.
@@ -86,35 +68,56 @@ class TestNvrtcArch(unittest.TestCase):
         self.assertRaises(
             compiler.CompileException, self._compile, '80')
 
+    @unittest.skipUnless(11000 <= cuda_version(),
+                         'Requires CUDA 11.0 or later')
+    def test_compile_cuda11(self):
+        # This test is intended to detect specification change in NVRTC API.
+
+        # It should not fail.
+        self._compile('80')
+
+        # It should fail.
+        self.assertRaises(
+            compiler.CompileException, self._compile, '83')
+
 
 @testing.gpu
 class TestNvrtcStderr(unittest.TestCase):
 
-    def test(self):
+    @unittest.skipIf(cupy.cuda.runtime.is_hip,
+                     'HIPRTC has different error message')
+    def test1(self):
         # An error message contains the file name `kern.cu`
-        with six.assertRaisesRegex(self, compiler.CompileException, 'kern.cu'):
+        with self.assertRaisesRegex(compiler.CompileException, 'kern.cu'):
             compiler.compile_using_nvrtc('a')
 
+    @unittest.skipIf(not cupy.cuda.runtime.is_hip,
+                     'NVRTC has different error message')
+    def test2(self):
+        with self.assertRaises(compiler.CompileException) as e:
+            compiler.compile_using_nvrtc('a')
+            assert "unknown type name 'a'" in e
+
 
 class TestIsValidKernelName(unittest.TestCase):
 
     def test_valid(self):
-        self.assertTrue(compiler.is_valid_kernel_name('valid_name_1'))
+        assert compiler.is_valid_kernel_name('valid_name_1')
 
     def test_empty(self):
-        self.assertFalse(compiler.is_valid_kernel_name(''))
+        assert not compiler.is_valid_kernel_name('')
 
     def test_start_with_digit(self):
-        self.assertFalse(compiler.is_valid_kernel_name('0_invalid'))
+        assert not compiler.is_valid_kernel_name('0_invalid')
 
     def test_new_line(self):
-        self.assertFalse(compiler.is_valid_kernel_name('invalid\nname'))
+        assert not compiler.is_valid_kernel_name('invalid\nname')
 
     def test_symbol(self):
-        self.assertFalse(compiler.is_valid_kernel_name('invalid$name'))
+        assert not compiler.is_valid_kernel_name('invalid$name')
 
     def test_space(self):
-        self.assertFalse(compiler.is_valid_kernel_name('invalid name'))
+        assert not compiler.is_valid_kernel_name('invalid name')
 
 
 class TestExceptionPicklable(unittest.TestCase):
@@ -124,3 +127,12 @@ class TestExceptionPicklable(unittest.TestCase):
         e2 = pickle.loads(pickle.dumps(e1))
         assert e1.args == e2.args
         assert str(e1) == str(e2)
+
+
+class TestCompileWithCache:
+    def test_compile_module_with_cache(self):
+        compiler._compile_module_with_cache('__device__ void func() {}')
+
+    def test_deprecated_compile_with_cache(self):
+        with pytest.warns(UserWarning):
+            compiler.compile_with_cache('__device__ void func() {}')
