@@ -1,36 +1,41 @@
 import itertools
+import warnings
+
 import numpy
-import unittest
+import pytest
 
 import cupy
 from cupy import testing
 
 
 float_types = [numpy.float16, numpy.float32, numpy.float64]
-complex_types = [numpy.complex, numpy.complex64, numpy.complex128]
+complex_types = [numpy.complex64, numpy.complex128]
 signed_int_types = [numpy.int8, numpy.int16, numpy.int32, numpy.int64]
 unsigned_int_types = [numpy.uint8, numpy.uint16, numpy.uint32, numpy.uint64]
 int_types = signed_int_types + unsigned_int_types
-all_types = [numpy.bool] + float_types + int_types + complex_types
+all_types = [numpy.bool_] + float_types + int_types + complex_types
 negative_types = (
-    [numpy.bool] + float_types + signed_int_types + complex_types)
-negative_no_complex_types = [numpy.bool] + float_types + signed_int_types
-no_complex_types = [numpy.bool] + float_types + int_types
+    [numpy.bool_] + float_types + signed_int_types + complex_types)
+negative_types_wo_fp16 = (
+    [numpy.bool_] + [numpy.float32, numpy.float64]
+    + [numpy.int16, numpy.int32, numpy.int64] + complex_types)
+negative_no_complex_types = [numpy.bool_] + float_types + signed_int_types
+no_complex_types = [numpy.bool_] + float_types + int_types
 
 
 @testing.gpu
 @testing.parameterize(*(
     testing.product({
         'nargs': [1],
-        'name': ['reciprocal', 'conj', 'angle'],
+        'name': ['reciprocal', 'conj', 'conjugate', 'angle'],
     }) + testing.product({
         'nargs': [2],
         'name': [
             'add', 'multiply', 'divide', 'power', 'subtract', 'true_divide',
-            'floor_divide', 'fmod', 'remainder'],
+            'floor_divide', 'float_power', 'fmod', 'remainder'],
     })
 ))
-class TestArithmeticRaisesWithNumpyInput(unittest.TestCase):
+class TestArithmeticRaisesWithNumpyInput:
 
     def test_raises_with_numpy_input(self):
         nargs = self.nargs
@@ -43,7 +48,7 @@ class TestArithmeticRaisesWithNumpyInput(unittest.TestCase):
                 # We don't test all-cupy-array inputs here
                 continue
             arys = [xp.array([2, -3]) for xp in input_xp_list]
-            with self.assertRaises(TypeError):
+            with pytest.raises(TypeError):
                 func(*arys)
 
 
@@ -53,11 +58,18 @@ class TestArithmeticRaisesWithNumpyInput(unittest.TestCase):
         'arg1': ([testing.shaped_arange((2, 3), numpy, dtype=d)
                   for d in all_types
                   ] + [0, 0.0j, 0j, 2, 2.0, 2j, True, False]),
-        'name': ['conj', 'angle', 'real', 'imag'],
+        'name': ['conj', 'conjugate', 'real', 'imag'],
+    }) + testing.product({
+        'arg1': ([testing.shaped_arange((2, 3), numpy, dtype=d)
+                  for d in all_types
+                  ] + [0, 0.0j, 0j, 2, 2.0, 2j, True, False]),
+        'deg': [True, False],
+        'name': ['angle'],
     }) + testing.product({
         'arg1': ([numpy.array([-3, -2, -1, 1, 2, 3], dtype=d)
-                  for d in negative_types
+                  for d in negative_types_wo_fp16
                   ] + [0, 0.0j, 0j, 2, 2.0, 2j, -2, -2.0, -2j, True, False]),
+        'deg': [True, False],
         'name': ['angle'],
     }) + testing.product({
         'arg1': ([testing.shaped_arange((2, 3), numpy, dtype=d) + 1
@@ -66,28 +78,31 @@ class TestArithmeticRaisesWithNumpyInput(unittest.TestCase):
         'name': ['reciprocal'],
     })
 ))
-class TestArithmeticUnary(unittest.TestCase):
+class TestArithmeticUnary:
 
     @testing.numpy_cupy_allclose(atol=1e-5)
     def test_unary(self, xp):
         arg1 = self.arg1
         if isinstance(arg1, numpy.ndarray):
             arg1 = xp.asarray(arg1)
-        y = getattr(xp, self.name)(arg1)
 
-        is_over_1_13 = testing.numpy_satisfies('>=1.13.0')
-        if is_over_1_13 and self.name in ('real', 'imag'):
-            # From NumPy>=1.13, some functions return Python scalars for Python
-            # scalar inputs.
+        if self.name in {'angle'}:
+            y = getattr(xp, self.name)(arg1, self.deg)
+        else:
+            y = getattr(xp, self.name)(arg1)
+
+        if self.name in ('real', 'imag'):
+            # Some NumPy functions return Python scalars for Python scalar
+            # inputs.
             # We need to convert them to arrays to compare with CuPy outputs.
             if xp is numpy and isinstance(arg1, (bool, int, float, complex)):
                 y = xp.asarray(y)
 
             # TODO(niboshi): Fix this
             # numpy.real and numpy.imag return Python int if the input is
-            # Python bool. CuPy should return an array of dtype.int32 or
-            # dtype.int64 (depending on the platform) in such cases, instead
-            # of an array of dtype.bool.
+            # Python bool. CuPy should return an array of dtype=int32 or
+            # dtype=int64 (depending on the platform) in such cases, instead
+            # of an array of dtype=bool.
             if xp is cupy and isinstance(arg1, bool):
                 y = y.astype(int)
 
@@ -98,7 +113,7 @@ class TestArithmeticUnary(unittest.TestCase):
     'shape': [(3, 2), (), (3, 0, 2)],
 }))
 @testing.gpu
-class TestComplex(unittest.TestCase):
+class TestComplex:
 
     @testing.for_all_dtypes(no_complex=True)
     @testing.numpy_cupy_array_equal()
@@ -190,20 +205,13 @@ class ArithmeticBinaryBase:
         dtype1 = np1.dtype
         dtype2 = np2.dtype
 
-        if self.name == 'power':
+        if self.name == 'power' or self.name == 'float_power':
             # TODO(niboshi): Fix this: power(0, 1j)
             #     numpy => 1+0j
             #     cupy => 0j
             if dtype2 in complex_types and (np1 == 0).any():
                 return xp.array(True)
 
-            # TODO(niboshi): Fix this: xp.power(0j, 0)
-            #     numpy => 1+0j
-            #     cupy => 0j
-            c_arg1 = dtype1 in complex_types
-            if c_arg1 and (np1 == 0j).any() and (np2 == 0).any():
-                return xp.array(True)
-
         # TODO(niboshi): Fix this: xp.add(0j, xp.array([2.], 'f')).dtype
         #     numpy => complex64
         #     cupy => complex128
@@ -216,17 +224,16 @@ class ArithmeticBinaryBase:
         if isinstance(arg2, numpy.ndarray):
             arg2 = xp.asarray(arg2)
 
-        # NumPy>=1.13.0 does not support subtraction between booleans
-        # TODO(niboshi): Write a separate test to check both NumPy and CuPy
-        # raise TypeError.
-        if testing.numpy_satisfies('>=1.13.0') and self.name == 'subtract':
-            if dtype1 == numpy.bool_ and dtype2 == numpy.bool_:
-                return xp.array(True)
+        # Subtraction between booleans is not allowed.
+        if (self.name == 'subtract'
+                and dtype1 == numpy.bool_
+                and dtype2 == numpy.bool_):
+            return xp.array(True)
 
         func = getattr(xp, self.name)
-        with testing.NumpyError(divide='ignore'):
-            with numpy.warnings.catch_warnings():
-                numpy.warnings.filterwarnings('ignore')
+        with numpy.errstate(divide='ignore'):
+            with warnings.catch_warnings():
+                warnings.filterwarnings('ignore')
                 if self.use_dtype:
                     y = func(arg1, arg2, dtype=self.dtype)
                 else:
@@ -235,7 +242,8 @@ class ArithmeticBinaryBase:
         # TODO(niboshi): Fix this. If rhs is a Python complex,
         #    numpy returns complex64
         #    cupy returns complex128
-        if xp is cupy and isinstance(arg2, complex):
+        if (xp is cupy and isinstance(arg2, complex)
+                and self.name != 'float_power'):
             if dtype1 in (numpy.float16, numpy.float32):
                 y = y.astype(numpy.complex64)
 
@@ -262,7 +270,7 @@ class ArithmeticBinaryBase:
         'arg2': [testing.shaped_reverse_arange((2, 3), numpy, dtype=d)
                  for d in all_types
                  ] + [0, 0.0, 0j, 2, 2.0, 2j, True, False],
-        'name': ['add', 'multiply', 'power', 'subtract'],
+        'name': ['add', 'multiply', 'power', 'subtract', 'float_power'],
     }) + testing.product({
         'arg1': [numpy.array([-3, -2, -1, 1, 2, 3], dtype=d)
                  for d in negative_types
@@ -273,7 +281,7 @@ class ArithmeticBinaryBase:
         'name': ['divide', 'true_divide', 'subtract'],
     })
 ))
-class TestArithmeticBinary(ArithmeticBinaryBase, unittest.TestCase):
+class TestArithmeticBinary(ArithmeticBinaryBase):
 
     def test_binary(self):
         self.use_dtype = False
@@ -297,7 +305,7 @@ class TestArithmeticBinary(ArithmeticBinaryBase, unittest.TestCase):
                  for d in float_types] + [0.0, 2.0, -2.0],
         'arg2': [numpy.array([-3, -2, -1, 1, 2, 3], dtype=d)
                  for d in float_types] + [0.0, 2.0, -2.0],
-        'name': ['power', 'true_divide', 'subtract'],
+        'name': ['power', 'true_divide', 'subtract', 'float_power'],
         'dtype': [numpy.float64],
         'use_dtype': [True, False],
     }) + testing.product({
@@ -322,16 +330,174 @@ class TestArithmeticBinary(ArithmeticBinaryBase, unittest.TestCase):
         'use_dtype': [True, False],
     })
 ))
-class TestArithmeticBinary2(ArithmeticBinaryBase, unittest.TestCase):
+class TestArithmeticBinary2(ArithmeticBinaryBase):
 
     def test_binary(self):
-        if (self.use_dtype and
-                numpy.lib.NumpyVersion(numpy.__version__) < '1.10.0'):
-            raise unittest.SkipTest('Test for numpy>=1.10')
         self.check_binary()
 
 
-class TestArithmeticModf(unittest.TestCase):
+class UfuncTestBase:
+
+    @testing.numpy_cupy_allclose(accept_error=TypeError)
+    def check_casting_out(self, in0_type, in1_type, out_type, casting, xp):
+        a = testing.shaped_arange((2, 3), xp, in0_type)
+        b = testing.shaped_arange((2, 3), xp, in1_type)
+        c = xp.zeros((2, 3), out_type)
+        if casting != 'unsafe':
+            # may raise TypeError
+            return xp.add(a, b, out=c, casting=casting)
+
+        with warnings.catch_warnings(record=True) as ws:
+            warnings.simplefilter('always')
+            ret = xp.add(a, b, out=c, casting=casting)
+        ws = [w.category for w in ws]
+        assert all([w == numpy.ComplexWarning for w in ws]), str(ws)
+        return ret, xp.array(len(ws))
+
+    @testing.numpy_cupy_allclose(accept_error=TypeError)
+    def check_casting_dtype(self, in0_type, in1_type, dtype, casting, xp):
+        a = testing.shaped_arange((2, 3), xp, in0_type)
+        b = testing.shaped_arange((2, 3), xp, in1_type)
+        if casting != 'unsafe':
+            # may raise TypeError
+            return xp.add(a, b, dtype=dtype, casting=casting)
+
+        with warnings.catch_warnings(record=True) as ws:
+            warnings.simplefilter('always')
+            ret = xp.add(a, b, dtype=dtype, casting='unsafe')
+        ws = [w.category for w in ws]
+        assert all([w == numpy.ComplexWarning for w in ws]), str(ws)
+        return ret, xp.array(len(ws))
+
+    # delete this, once check_casting_dtype passes
+    @testing.numpy_cupy_allclose()
+    def check_casting_dtype_unsafe_ignore_warnings(
+            self, in0_type, in1_type, dtype, xp):
+        a = testing.shaped_arange((2, 3), xp, in0_type)
+        b = testing.shaped_arange((2, 3), xp, in1_type)
+        with warnings.catch_warnings():
+            warnings.simplefilter('ignore')
+            return xp.add(a, b, dtype=dtype, casting='unsafe')
+
+
+class TestUfunc(UfuncTestBase):
+
+    @pytest.mark.parametrize('casting', [
+        pytest.param('no', marks=pytest.mark.skip('flaky xfail')),
+        pytest.param('equiv', marks=pytest.mark.skip('flaky xfail')),
+        'safe',
+        'same_kind',
+        'unsafe',
+    ])
+    @testing.for_all_dtypes_combination(
+        names=['in0_type', 'in1_type', 'out_type'], full=False)
+    def test_casting_out(self, in0_type, in1_type, out_type, casting):
+        self.check_casting_out(in0_type, in1_type, out_type, casting)
+
+    @pytest.mark.xfail()
+    @pytest.mark.parametrize('casting', [
+        'no',
+        'equiv',
+    ])
+    @pytest.mark.parametrize(('in0_type', 'in1_type', 'out_type'), [
+        (numpy.int16, numpy.int32, numpy.int32),
+    ])
+    def test_casting_out_xfail1(self, in0_type, in1_type, out_type, casting):
+        self.check_casting_out(in0_type, in1_type, out_type, casting)
+
+    @pytest.mark.skip('flaky xfail')
+    @pytest.mark.parametrize('casting', [
+        'no',
+        'equiv',
+        'safe',
+        'same_kind',
+        'unsafe',
+    ])
+    @testing.for_all_dtypes_combination(
+        names=['in0_type', 'in1_type', 'dtype'], full=False)
+    def test_casting_dtype(self, in0_type, in1_type, dtype, casting):
+        self.check_casting_dtype(in0_type, in1_type, dtype, casting)
+
+    @pytest.mark.xfail()
+    @pytest.mark.parametrize('casting', [
+        'no',
+        'equiv',
+    ])
+    @pytest.mark.parametrize(('in0_type', 'in1_type', 'dtype'), [
+        (numpy.int16, numpy.int32, numpy.int32),
+    ])
+    def test_casting_dtype_xfail1(self, in0_type, in1_type, dtype, casting):
+        self.check_casting_dtype(in0_type, in1_type, dtype, casting)
+
+    @pytest.mark.xfail()
+    @pytest.mark.parametrize('casting', [
+        'no',
+        'equiv',
+        'safe',
+        'same_kind',
+    ])
+    @pytest.mark.parametrize(('in0_type', 'in1_type', 'dtype'), [
+        (numpy.int32, numpy.int32, numpy.bool_),
+        (numpy.float64, numpy.float64, numpy.int32),
+    ])
+    def test_casting_dtype_xfail2(self, in0_type, in1_type, dtype, casting):
+        self.check_casting_dtype(in0_type, in1_type, dtype, casting)
+
+    @pytest.mark.xfail()
+    @pytest.mark.parametrize(('in0_type', 'in1_type', 'dtype'), [
+        (numpy.complex64, numpy.complex64, numpy.float32),
+    ])
+    def test_casting_dtype_xfail3(self, in0_type, in1_type, dtype):
+        casting = 'unsafe'
+        self.check_casting_dtype(in0_type, in1_type, dtype, casting)
+
+    @testing.for_all_dtypes_combination(
+        names=['in0_type', 'in1_type', 'dtype'], full=False)
+    def test_casting_dtype_unsafe_ignore_warnings(
+            self, in0_type, in1_type, dtype):
+        self.check_casting_dtype_unsafe_ignore_warnings(
+            in0_type, in1_type, dtype
+        )
+
+
+@testing.slow
+class TestUfuncSlow(UfuncTestBase):
+
+    @pytest.mark.parametrize('casting', [
+        pytest.param('no', marks=pytest.mark.xfail()),
+        pytest.param('equiv', marks=pytest.mark.xfail()),
+        'safe',
+        'same_kind',
+        'unsafe',
+    ])
+    @testing.for_all_dtypes_combination(
+        names=['in0_type', 'in1_type', 'out_type'], full=True)
+    def test_casting_out(self, in0_type, in1_type, out_type, casting):
+        self.check_casting_out(in0_type, in1_type, out_type, casting)
+
+    @pytest.mark.xfail()
+    @pytest.mark.parametrize('casting', [
+        'no',
+        'equiv',
+        'safe',
+        'same_kind',
+        'unsafe',
+    ])
+    @testing.for_all_dtypes_combination(
+        names=['in0_type', 'in1_type', 'dtype'], full=True)
+    def test_casting_dtype(self, in0_type, in1_type, dtype, casting):
+        self.check_casting_dtype(in0_type, in1_type, dtype, casting)
+
+    @testing.for_all_dtypes_combination(
+        names=['in0_type', 'in1_type', 'dtype'], full=True)
+    def test_casting_dtype_unsafe_ignore_warnings(
+            self, in0_type, in1_type, dtype):
+        self.check_casting_dtype_unsafe_ignore_warnings(
+            in0_type, in1_type, dtype
+        )
+
+
+class TestArithmeticModf:
 
     @testing.for_float_dtypes()
     @testing.numpy_cupy_allclose()
@@ -342,3 +508,19 @@ class TestArithmeticModf(unittest.TestCase):
         d[0] = b
         d[1] = c
         return d
+
+
+@testing.parameterize(*testing.product({
+    'xp': [numpy, cupy],
+    'shape': [(3, 2), (), (3, 0, 2)]
+}))
+@testing.gpu
+class TestBoolSubtract:
+
+    def test_bool_subtract(self):
+        xp = self.xp
+        shape = self.shape
+        x = testing.shaped_random(shape, xp, dtype=numpy.bool_)
+        y = testing.shaped_random(shape, xp, dtype=numpy.bool_)
+        with pytest.raises(TypeError):
+            xp.subtract(x, y)
